package build

import (
	"git.akyoto.dev/cli/q/src/build/asm"
	"git.akyoto.dev/cli/q/src/build/ast"
	"git.akyoto.dev/cli/q/src/build/config"
	"git.akyoto.dev/cli/q/src/build/cpu"
	"git.akyoto.dev/cli/q/src/build/expression"
)

// ExpressionToRegister moves the result of an expression into the given register.
func (f *Function) ExpressionToRegister(root *expression.Expression, register cpu.Register) error {
	if config.Verbose {
		f.Logf("%s to register %s", root, register)
	}

	operation := root.Token

	if root.IsLeaf() {
		return f.TokenToRegister(operation, register)
	}

	if ast.IsFunctionCall(root) {
		err := f.CompileFunctionCall(root)

		if err != nil {
			return err
		}

		if register != f.cpu.Return[0] {
			f.assembler.RegisterRegister(asm.MOVE, register, f.cpu.Return[0])
		}

		return nil
	}

	left := root.Children[0]
	right := root.Children[1]

	err := f.ExpressionToRegister(left, register)

	if err != nil {
		return err
	}

	f.SaveRegister(register)
	return f.Execute(operation, register, right)
}
